//*****************************************************************************
//*****************************************************************************
//  FILENAME:   sensor.c
//  Version: 1.0, 2007/03/08
//  Generated by Andy Kiser
//
//  DESCRIPTION:  sensor specific functions 
//
//-----------------------------------------------------------------------------
// Originally authored by Toradex AG. This work is now available under the terms and conditions
// of the Creative Commons License 'Attribution CC BY', details of which can be found here:
// http://creativecommons.org/licenses/by/3.0/
//*****************************************************************************
//*****************************************************************************
//#define __DEBUG__
#include <m8c.h>        // part specific constants and macros
#include "sensor.h"
#include "psocgpioint.h"
#include "spim.h"
#include "feature.h"
#include "parameters.h"
#include "types_n_macros.h"
//----------------------------------------
// Sensor parameters
// Factory Defaults are defined here (NUM_xxx are defined in "sensor.h")
//----------------------------------------
const BYTE FactoryDefault_Param_8   [NUM_PARAM_8] =
{
	0x00,		// report mode
	0x00		// LED mode
};

const WORD FactoryDefault_Param_16  [NUM_PARAM_16] = 
{
	500,		// report rate [ms]
	500,		// sample rate [ms]
	50000,		// sample rate [us]	(not used for this sensor)
};

const DWORD FactoryDefault_Param_32  [NUM_PARAM_32] =
{
	0x00000000	// currently no 32bit parameters
};

const tParam_168 FactoryDefault_Param_168 [NUM_PARAM_168] =
{
	"4-20mA isolated",	// user device name
	"Frame Number",		// user name Channel 0
	"4-20mA",			// user name Channel 1
};


//----------------------------------------
// this module's global variables
//----------------------------------------
tReport Report;

//----------------------------------------
// constant lookup-table for ADC->physical value conversion
//----------------------------------------


//----------------------------------------
// type definitions
//----------------------------------------

typedef enum 
{	
	eReset,
	eIdle,
	eMeasuring,
	eReadADC
} tFSM_Measure;

//----------------------------------------
// variables

tFSM_Measure	FSM;
WORD			Countdown_1ms;

BOOL			NewSample = FALSE;

// store original drivemodes during suspend
BYTE			DM0_2, DM0_1, DM0_0;
BYTE			DM2_2, DM2_1, DM2_0;

//-----------------------------------------------------------------------------
// Initializes the Sensor 
//-----------------------------------------------------------------------------
void Sensor_Start ()
{
	FSM = eReset;

	// SCK idle state = 1, to conserve power
	// ATTENTION: during each falling edge of nCS, SCK *must* be *low*, to 
	//            activate external SCK mode !!!
	SPIM_Start(SPIM_SPIM_MODE_1 | SPIM_SPIM_MSB_FIRST);
	// nCS idle state = 1
	GPIO_HIGH(nCS)
	
	// turn on power for the sensor
	GPIO_LOW(OFF_nON)
}
//-----------------------------------------------------------------------------
// Prepare Sensor for Suspend
//-----------------------------------------------------------------------------
void Sensor_Suspend()
{
	// turn off power for the sensor
	GPIO_HIGH(OFF_nON)

	// store original drivemodes during  suspend
	DM0_2 = PRT0DM2;
	DM0_1 = PRT0DM1;
	DM0_0 = PRT0DM0;
	DM2_2 = PRT2DM2;
	DM2_1 = PRT2DM1;
	DM2_0 = PRT2DM0;
	
	// set all I/Os to/from the sensor to High-Z Analog (=110), 
	// except the OFF_nON signal. 
	// OFF_nON is on Port0, all other signals on Port2
	PRT0DM2 |= ~OFF_nON_MASK;
	PRT0DM1 |= ~OFF_nON_MASK;
	PRT0DM0 &=  OFF_nON_MASK;
	PRT2DM2  = 0xFF;
	PRT2DM1  = 0xFF;
	PRT2DM0  = 0x00;

}
//-----------------------------------------------------------------------------
// Resume Sensor from Suspend mode
//-----------------------------------------------------------------------------
void Sensor_Resume()
{
	BYTE ReadMode;
	
	// restore drivemodes for GPIOs to/from the sensor
	PRT2DM2 = DM2_2;
	PRT2DM1 = DM2_1;
	PRT2DM0 = DM2_0;

	PRT0DM2 = DM0_2;
	PRT0DM1 = DM0_1;
	PRT0DM0 = DM0_0;

	// start up sensor again
	Sensor_Start();
}

//-----------------------------------------------------------------------------
// Read Raw ADC value from the sensor
// The function assumes the following (idle) conditions on entry:
// 		CSn == 0
//		SCK == 0
//		MISO== 0 (indicating that conversion has finished)
// 		SPI-Mode == 0
// Side Effect
//-----------------------------------------------------------------------------
void Sensor_ReadADC (WORD *CurrentRaw, BYTE *Status)
{
	signed char i;
	union
	{
		BYTE b[4];
		unsigned long ul;
	} Rx;
	
	// -- dummy read to clear RX_BUFFER_FULL flag
	SPIM_bReadRxData(); 
	
	// -- receive 3+16+5 bits from the Sensor
	for (i=1; i<=3; i++)
	{		// read one byte
		SPIM_SendTxData( 0x00);
		while( !( SPIM_bReadStatus() & SPIM_SPIM_RX_BUFFER_FULL ) );   	
		Rx.b[i] = SPIM_bReadRxData();
	}
	Rx.b[0] = 0;

	// -- prepare return values
	Rx.ul <<= 3;
		// 3 Bit Status
	*Status     = Rx.b[0];
		// 16 bit ADC value
	*CurrentRaw = 256*Rx.b[1] + Rx.b[2];
}

//-----------------------------------------------------------------------------
// Function that will be called when a FEATURE REPORT is received that 
// cannot be handled with the standard functionality in feature.c
// The feature report data is available in the "FEATURE_RPT" variable
//
// --> see feature.c for an example how to parse FEATURE_RPT
//
//	return Value: number of Parameter bytes (1-31) that are 
//				  returned in the IN report
//-----------------------------------------------------------------------------
BYTE Sensor_Specific_Feature(void)
{
	// currently no special feature reports are handled
	return 0;
}

//-----------------------------------------------------------------------------
// Function that will be called when a FEATURE REPORT is received that 
// sets / gets a 8/16/32/168bit parameter in feature.c
// The feature report data is available in the "FEATURE_RPT" variable
//
// --> see feature.c for an example how to parse FEATURE_RPT
//-----------------------------------------------------------------------------
void Sensor_Specific_Parameter(BYTE *NumberOfReturnedBytes)
{
	// currently there are no specific parameters
}

//-----------------------------------------------------------------------------
// void Sensor_FSM_Measure(void)
//
//	finite state machine to control the measurement process for 
//  the Sensor
//
//  IMPORTANT: 	call this routine once every millisecond. Otherwise the timing
//				parameters cannot be met.
//-----------------------------------------------------------------------------
void Sensor_FSM_Measure() 
{
	WORD ADC;
	BYTE Status;
	char i;
	
	union
	{
		BYTE			b[4];
		WORD			w[2];
		unsigned long 	ul;
	} Current;

	// decrement millisecond counter
	if (Countdown_1ms != 0)
		Countdown_1ms--;

	// process actual state machine
	switch (FSM)
	{
		case eReset:
			// ADC automatically starts conversion on Power-on Reset (POR)
			FSM = eMeasuring;
			break;
			
		case eMeasuring:
			if (Countdown_1ms == 151)
			{
				// Activate SPI and bring SCK signal to low
				SPIM_Start(SPIM_SPIM_MODE_0 | SPIM_SPIM_MSB_FIRST);
				
				// activate nCS to be able to detect EOC flag
				GPIO_LOW(nCS)  
				for(i=0; i<10;i++);  // short delay to allow signal to settle
				if ( (MISO_Data_ADDR & MISO_MASK) == 0 )
				{	
					// Countdown_1ms == 151, enough time to do one more conversion, before result is requested
					// => generate at least one clock cycle
					SPIM_bReadRxData(); 	// dummy read to clear RX_BUFFER_FULL flag
					SPIM_SendTxData( 0x00);
					while( !( SPIM_bReadStatus() & SPIM_SPIM_RX_BUFFER_FULL ) );   	
				}
				// deactivate nCS to start a new conversion
				GPIO_HIGH(nCS)
				SPIM_Start(SPIM_SPIM_MODE_1 | SPIM_SPIM_MSB_FIRST);
			}
			if (Countdown_1ms <= 151)
				FSM = eReadADC;
			break;

		case eReadADC:
			if (Countdown_1ms == 0)
			{
				// Activate SPI and bring SCK signal to low
				SPIM_Start(SPIM_SPIM_MODE_0 | SPIM_SPIM_MSB_FIRST);
			
				GPIO_LOW(nCS)
				for(i=0; i<10;i++);  // short delay to allow signal to settle
				if ( (MISO_Data_ADDR & MISO_MASK) == 0 )
				{
					// reset samplerate counter
					Get_Param_16(PAR16_SAMPLERATE_MS, VOLATILE, &Countdown_1ms);
					Countdown_1ms -=3; // offset for processing that takes longer than 1ms
					FSM = eMeasuring;

					// reading the ADC automatically starts a new conversion.
					Sensor_ReadADC (&Current.w[0], &Status);
					Current.w[1] = 0;

					// Calculate Current  0...65536 -> 0...20.48mA -> 0...20480
					Current.ul >>= 1;
					Current.ul += Current.ul >>2;
					// fill data into IN report

					Report.Ch1_LSB = Current.b[1];
					Report.Ch1_MSB =  Current.b[0];
					NewSample = TRUE;
				}
				GPIO_HIGH(nCS)
				SPIM_Start(SPIM_SPIM_MODE_1 | SPIM_SPIM_MSB_FIRST); // only to bring SCK to high
			}
			break;			
	}
}

//-----------------------------------------------------------------------------
void Sensor_Debug(void)
{
/* currently no debug function */
}